<?php

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Webtexttool_Social
{

    /**
     * The ID of this plugin.
     *
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    private $title = null;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string $plugin_name The name of this plugin.
     * @param      string $version The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Set the start string
     *
     * @return string
     */
    public function set_start()
    {
        return "\n<!-- Webtexttool SEO Plugin " . WTT_VERSION . " - https://www.webtexttool.com/ -->\n";
    }

    /**
     * Set the end string
     *
     * @return string
     */
    public function set_end()
    {
        return "\n<!-- /Webtexttool SEO Plugin -->\n";
    }

    /**
     * Insert the social meta tags in the header
     */
    public function set_header_meta()
    {
        $data = "";
        $wtt_social = get_option("wtt_social");
        $data .= $this->set_start();

        if (isset($wtt_social['canonical_url']) && $wtt_social['canonical_url'] == "on") {
            remove_action('wp_head', 'rel_canonical');
            $data .= $this->get_canonical_url();
        }

        if (isset($wtt_social['show_meta_desc']) && $wtt_social['show_meta_desc'] == "on" && $this->get_other_plugin_description()) {
            $data .= $this->get_meta_description();
        }

        if (isset($wtt_social['socialmetabox']) && $wtt_social['socialmetabox'] == "on") {
            if (isset($wtt_social['opengraph']) && $wtt_social['opengraph'] == "on") {
                $data .= $this->get_opengraph_meta();
            }

            if (isset($wtt_social['twitter']) && $wtt_social['twitter'] == "on") {
                $data .= $this->get_twittercard_meta();
            }
        }

        $data .= $this->set_end();
        echo $data;
    }

    /**
     * Default get_post_meta function
     *
     * @param $meta
     * @param int $post_id
     * @return bool|string
     */
    private function getPostMeta($meta, $post_id = 0)
    {
        global $post;

        $post_meta = htmlspecialchars(get_post_meta($post_id ? $post_id : $post->ID, $meta, true), ENT_QUOTES);

        if (!empty($post_meta)) {
            return $post_meta;
        }

        return "";
    }
    /**
     * Set the SEO title by add_filter (called in Webtexttool::define_social_hooks)
     *
     * @param $title
     * @param string $separator
     * @param string $sep_location
     * @return null|string|Webtexttool_Social
     */
    public function wtt_set_seo_title($title, $separator = '', $sep_location = '')
    {
        if (is_null($this->title)) {
            $this->title = $this->wtt_generate_seo_title($title, $sep_location);
        }

        return $this->title;
    }

    /**
     * Get wtt_settings option variable
     *
     * @param string $index The option variable to get the settings from
     * @param object|array $object The object variable
     *
     * @return string
     */
    private function get_wtt_title_from_options($index, $object = array())
    {
        $wtt_settings = get_option("wtt_settings");

        if (!isset($wtt_settings[$index]) || $wtt_settings[$index] === '') {
            if (is_singular()) {
                return $this->replacePatterns('{{post_title}} {{sep}} {{sitetitle}}', $object);
            }

            return '';
        }

        return $this->replacePatterns($wtt_settings[$index], $object);
    }

    /**
     * Generates the SEO title based on input
     *
     * @param $title
     * @param $sep_location
     * @return string|Webtexttool_Social
     */
    private function wtt_generate_seo_title($title, $sep_location)
    {
        if (is_feed()) {
            return $title;
        }

        $separator = $this->replacePatterns('{{sep}}', array());
        $separator = ' ' . trim($separator) . ' ';

        if (trim($sep_location) === '') {
            $sep_location = (is_rtl()) ? 'left' : 'right';
        }

        $original_title = $title;

        $modified_title = true;

        $title_part = '';

        if (is_front_page() && get_option('show_on_front') === 'page' && is_page(get_option('page_on_front'))) { //STATIC HOMEPAGE (frontend)
            $title = $this->get_wtt_static_page_title();
        } elseif (is_home() && get_option('show_on_front') === 'posts') {              //LATEST POSTS (default)
            $title = $this->get_wtt_title_from_options('title-home-wtt');
        } elseif ($this->is_single_page()) {                                            //STATIC POST PAGE (frontend)
            $post = get_post($this->get_single_page_id());
            $title = $this->get_wtt_static_page_title($post);

            if (!is_string($title) || $title === '') {
                $title_part = $original_title;
            }
        } elseif (is_search()) {                                                        // Search page
            $title = $this->get_wtt_title_from_options('title-search-wtt');

            if (!is_string($title) || $title === '') {
                $title_part = $original_title;
            }
        } elseif (is_tax() || is_category() || is_tag()) {                            // Taxonomy title
            $title = $this->get_wtt_taxonomy_title();

            if (!is_string($title) || $title === '') {
                if (is_category()) {
                    $title_part = single_cat_title('', false);
                } elseif (is_tag()) {
                    $title_part = single_tag_title('', false);
                } else {
                    $title_part = single_term_title('', false);
                    if ($title_part === '') {
                        $term = get_queried_object() === null ? get_queried_object() : '';
                        $title_part = $term->name;
                    }
                }
            }
        } elseif (is_author()) {
            $title = $this->get_wtt_author_title();

            if (!is_string($title) || $title === '') {
                $title_part = get_the_author_meta('display_name', get_query_var('author'));
            }
        } elseif ($this->is_woocommerce_page()) { // WooCommerce Shop page
            $post = get_post(function_exists('wc_get_page_id') ? wc_get_page_id('shop') : (-1));
            $title = $this->get_wtt_title($post);

            if (!is_string($title) || $title === '') {
                $title = $this->get_wtt_post_type_archive_title($separator, $sep_location);
            }
        } elseif (is_post_type_archive()) {
            $title = $this->get_wtt_post_type_archive_title($separator, $sep_location);
        } elseif (is_archive()) { // if any type of Archive page is being displayed
            $title = $this->get_wtt_title_from_options('title-archive-wtt');

            if (empty($title)) {
                $title_part = __('Archives', 'webtexttool');
            }
        } elseif (is_404()) {
            $title = $this->get_wtt_title_from_options('title-404-wtt');

            if (empty($title)) {
                $title_part = __('Page not found', 'webtexttool');
            }
        } else {
            $title_part = $title;
        }

        if (!empty($title_part) || (empty($title) && $modified_title)) {
            $title = $this->get_default_page_title($separator, $sep_location, $title_part);
        }

        return esc_html(wp_strip_all_tags(stripslashes($title), true));
    }

    private function is_single_page()
    {
        return $this->get_single_page_id() > 0;
    }

    private function get_single_page_id()
    {
        if (is_singular()) {
            return get_the_ID();
        }

        if (is_home() && get_option('show_on_front') === 'page') {
            return get_option('page_for_posts');
        }

        return 0;
    }

    private function get_wtt_author_title()
    {
        $author_id = get_query_var('author');
        $title = trim(get_the_author_meta('wtt_title', $author_id));

        if ($title !== '') {
            return $this->replacePatterns($title, array());
        }

        return $this->get_wtt_title_from_options('title-author-wtt');
    }

    private function get_wtt_taxonomy_title()
    {
        $queried_object = $GLOBALS['wp_query']->get_queried_object();

        $title = $this->get_wtt_tax_term_meta($queried_object, $queried_object->taxonomy, '_wtt_term_title');

        if (is_string($title) && $title !== '') {
            return $this->replacePatterns($title, $queried_object);
        }

        return $this->get_wtt_title_from_options('title-tax-' . $queried_object->taxonomy, $queried_object);
    }

    private function get_wtt_meta_without_term($meta) {
        $term = $GLOBALS['wp_query']->get_queried_object();

        return self::get_wtt_tax_term_meta( $term, $term->taxonomy, $meta );
    }

    private function get_wtt_tax_term_meta($term, $tax, $meta = null)
    {
        if (is_int($term)) {
            $term = get_term_by('id', $term, $tax);
        } elseif (is_string($term)) {
            $term = get_term_by('slug', $term, $tax);
        }

        if (isset($term->term_id) && is_object($term)) {
            $term_id = $term->term_id;
        } else {
            return false;
        }

        $wtt_tax_meta = $this->get_tax_meta($term_id, $meta);

        return $wtt_tax_meta;
    }

    private function get_tax_meta($term_id, $meta)
    {
        return get_term_meta($term_id, $meta, true);
    }

    private function get_wtt_post_type_archive_title($separator, $sep_location)
    {
        $post_type = get_query_var('post_type');

        $title = $this->get_wtt_title_from_options('title-ptarchive-' . $post_type);

        if (!is_string($title) || $title === '') {
            $post_type_obj = get_post_type_object($post_type);
            $title_part = '';

            if (isset($post_type_obj->labels->menu_name)) {
                $title_part = $post_type_obj->labels->menu_name;
            } elseif (isset($post_type_obj->name)) {
                $title_part = $post_type_obj->name;
            }

            $title = $this->get_default_page_title($separator, $sep_location, $title_part);
        }

        return $title;
    }

    private function get_default_page_title($sep, $sep_location, $title = '')
    {
        if ($sep_location === 'right') {
            $regex = '`\s*' . preg_quote(trim($sep), '`') . '\s*`u';
        } else {
            $regex = '`^\s*' . preg_quote(trim($sep), '`') . '\s*`u';
        }
        $title = preg_replace($regex, '', $title);

        if (!is_string($title) || (is_string($title) && $title === '')) {
            $title = wp_strip_all_tags(get_bloginfo('name'), true);
            $title = $this->add_page_number_to_title($sep, $sep_location, $title);
            $title = $this->add_title_part_to_title($sep, $sep_location, $title, wp_strip_all_tags(get_bloginfo('description'), true));

            return $title;
        }

        $title = $this->add_page_number_to_title($sep, $sep_location, $title);
        $title = $this->add_title_part_to_title($sep, $sep_location, $title, wp_strip_all_tags(get_bloginfo('name'), true));

        return $title;
    }

    private function add_page_number_to_title($sep, $sep_location, $title)
    {
        global $wp_query;

        if (!empty($wp_query->query_vars['paged']) && $wp_query->query_vars['paged'] > 1) {
            return $this->add_title_part_to_title($sep, $sep_location, $title, $wp_query->query_vars['paged'] . '/' . $wp_query->max_num_pages);
        }

        return $title;
    }

    private function add_title_part_to_title($sep, $sep_location, $title, $title_part)
    {
        if ($sep_location === 'right') {
            return $title . $sep . $title_part;
        }

        return $title_part . $sep . $title;
    }

    /**
     * Generate the title for static home page type
     *
     * @param null $queried_object
     * @return mixed
     */
    private function get_wtt_static_page_title($queried_object = null)
    {
        if ($queried_object === null) {
            $queried_object = $GLOBALS['wp_query']->get_queried_object();
        }

        $title = $this->get_wtt_title($queried_object);

        if ($title !== '') {
            return $title;
        };

        $post_type = (isset ($queried_object->post_type) ? $queried_object->post_type : $queried_object->query_var);

        return $this->get_wtt_title_from_options('title-' . $post_type, $queried_object);
    }

    /**
     * Test current if the current page is a shop page
     *
     * @return bool
     *
     */
    public function is_woocommerce_page() {
        if ( function_exists( 'is_shop' ) && function_exists( 'wc_get_page_id' ) ) {
            return is_shop() && ! is_search();
        }

        return false;
    }

    /**
     * Get the title from the custom wtt post meta
     *
     * @param null $queried_object
     * @return bool|string
     */
    private function get_wtt_title($queried_object = null)
    {
        if ($queried_object === null) {
            $queried_object = $GLOBALS['wp_query']->get_queried_object();
        }

        if (!is_object($queried_object)) {
            return $this->get_wtt_title_from_options('title-404-wtt');
        }

        $title = $this->getPostMeta('_wtt_post_title', $queried_object->ID);

        if ($title !== '') {
            return $this->replacePatterns($title, $queried_object);
        }

        return $title;
    }

    /**
     * Replace the dynamic patterns in a string
     *
     * @param string $string String to compare and replace
     * @param array $object Queried object or post type
     * @return string           The replaced string
     */
    private function replacePatterns($string, $object)
    {
        $replacement = new WTT_Replace_Patterns();

        return $replacement->replace($string, $object);
    }

    /**
     * Get the Open Graph Meta
     */
    private function get_opengraph_meta()
    {
        global $post;

        $meta = "";
        $wtt_social = get_option("wtt_social");

        $openGraphTitle = $this->generate_og_title();
        $openGraphImage = $this->get_custom_image();
        $ogDescription = $this->get_og_description();

        $meta .= sprintf('<meta property="og:url" content="%s" />', $this->get_og_url()) . "\n";

        $meta .= (($openGraphImage <> '') ? sprintf('<meta property="og:image" content="%s" />', $openGraphImage) . "\n" : '');
        $meta .= (($openGraphTitle <> '') ? sprintf('<meta property="og:title" content="%s" />', $openGraphTitle) . "\n" : '');
        $meta .= (($ogDescription <> '') ? sprintf('<meta property="og:description" content="%s" />', $ogDescription) . "\n" : '');

        $meta .= ((get_bloginfo('name') <> '') ? sprintf('<meta property="og:site_name" content="%s" />', get_bloginfo('name')) . "\n" : '');

        $meta .= sprintf('<meta property="og:locale" content="%s" />', isset($wtt_social['wtt_og_locale']) ? $wtt_social['wtt_og_locale'] : get_locale()) . "\n";

        if (is_author()) {
            $author = get_queried_object();

            $meta .= sprintf('<meta property="og:type" content="%s" />', 'profile') . "\n";
            if(get_the_author_meta('first_name', $author->ID) || get_the_author_meta('last_name', $author->ID) !== '') {
                $meta .= sprintf('<meta property="profile:first_name" content="%s" />', get_the_author_meta('first_name', $author->ID)) . "\n";
                $meta .= sprintf('<meta property="profile:last_name" content="%s" />', get_the_author_meta('last_name', $author->ID)) . "\n";
            }
        } else if (function_exists('is_product') && is_product()) {
            $meta .= sprintf('<meta property="og:type" content="%s" />', 'product') . "\n";

            $cat = get_the_terms($post->ID, 'product_cat');
            if (!empty($cat) && count($cat) > 0) {
                $meta .= sprintf('<meta property="product:category" content="%s" />', $cat[0]->name) . "\n";
            }
        } else if (is_front_page() || is_home()) {
            $meta .= sprintf('<meta property="og:type" content="%s" />', 'website') . "\n";
        } else if (is_singular()) {

            $meta .= sprintf('<meta property="og:type" content="%s" />', 'article') . "\n";
            $meta .= sprintf('<meta property="article:published_time" content="%s" />', get_the_time('c', $post->ID)) . "\n";
            $facebook = get_the_author_meta( 'facebook', $GLOBALS['post']->post_author );

            $meta .= (($facebook <> '') ? sprintf('<meta property="article:author" content="%s" />', $facebook) . "\n" : '');
            $meta .= (($wtt_social['facebook-site'] <> '') ? sprintf('<meta property="article:publisher" content="https://facebook.com/%s" />', $wtt_social['facebook-site']) . "\n" : '');

            $category = get_the_category($post->ID);
            if (!empty($category) && $category[0]->cat_name <> 'Uncategorized') {
                $meta .= sprintf('<meta property="article:section" content="%s" />', $category[0]->cat_name) . "\n";
            }

            $tags = get_the_tags();
            if ( ( is_array( $tags ) && !is_wp_error($tags) && $tags !== array())) {
                foreach ( $tags as $tag ) {
                    $meta .= sprintf('<meta property="article:tag" content="%s" />', $tag->name) . "\n";
                }
            }
        } else {
            $meta .= sprintf('<meta property="og:type" content="%s" />', 'object') . "\n";
        }

        return $meta;
    }

    /**
     * Get the Twitter Card Meta
     */
    private function get_twittercard_meta()
    {
        $meta = "";
        $wtt_social = get_option("wtt_social");

        $openGraphTitle = $this->generate_og_title();
        $openGraphImage = $this->get_custom_image();
        $ogDescription = $this->get_og_description();

        if (isset($wtt_social['twitter_card_type']) && $wtt_social['twitter_card_type'] <> '') {
            $meta .= sprintf('<meta name="twitter:card" content="%s" />', $wtt_social['twitter_card_type']) . "\n";
        }

        if ( is_singular() ) {
            $twitter = get_the_author_meta('twitter', $GLOBALS['post']->post_author);
            if (get_post() && is_object(get_post())) {
                $meta .= (($twitter <> '') ? sprintf('<meta name="twitter:creator" content="%s" />', $this->get_twitter_account($twitter)) . "\n" : '');
            }
        }

        $meta .= (($wtt_social['twitter-site'] <> '') ? sprintf('<meta name="twitter:site" content="%s" />', $this->get_twitter_account($wtt_social['twitter-site'])) . "\n" : '');
        $meta .= (($openGraphImage <> '') ? sprintf('<meta name="twitter:image" content="%s" />', $openGraphImage) . "\n" : '');
        $meta .= (($openGraphTitle <> '') ? sprintf('<meta name="twitter:title" content="%s" />', $openGraphTitle) . "\n" : '');
        $meta .= (($ogDescription <> '') ? sprintf('<meta name="twitter:description" content="%s" />', $ogDescription) . "\n" : '');

        $meta .= ((get_bloginfo('name') <> '') ? sprintf('<meta name="twitter:domain" content="%s" />', get_bloginfo('name')) : '');

        return $meta;
    }

    /**
     * Get the twitter account from the database
     *
     * @param string $twitterAccount
     * @return bool|string
     */
    private function get_twitter_account($twitterAccount)
    {
        if ($twitterAccount <> '') {
            if (strpos($twitterAccount, 'twitter.com') !== false) {
                preg_match('/twitter.com\/([@1-9a-z_-]+)/i', $twitterAccount, $matchResult);
                if (isset($matchResult[1]) && !empty($matchResult[1])) {
                    return '@' . str_replace('@', '', $matchResult[1]);
                }
            } else {
                preg_match('/([@1-9a-z_-]+)/i', $twitterAccount, $matchResult);
                if (isset($matchResult[1]) && !empty($matchResult[1])) {
                    return '@' . str_replace('@', '', $matchResult[1]);
                }
            }
        } else {
            return '';
        }
        return false;
    }

    /**
     * Checks if other plugins have description enabled
     *
     * @return bool
     */
    private function get_other_plugin_description()
    {
        if ($this->is_wp_seo_active()) {
            $wpseo = WPSEO_Frontend::get_instance();

            if ($wpseo->metadesc(false) !== '') {
                return false;
            } else {
                return true;
            }
        } else {
            return true;
        }
    }

    /**
     * Return part of a text based on min and max value
     *
     * @param $text
     * @param $min
     * @param $max
     * @return string
     */
    public function truncateDesc($text, $min = 100, $max = 110) {
        if ($text <> '' && strlen($text) > $max) {
            if (function_exists('strip_tags')) {
                $text = strip_tags($text);
            }
            $text = str_replace(']]>', ']]&gt;', $text);
            $text = @preg_replace('/\[(.+?)\]/is', '', $text);
            $text = strip_tags($text);

            if ($max < strlen($text)) {
                while ($text[$max] != ' ' && $max > $min) {
                    $max--;
                }
            }
            $text = substr($text, 0, $max);
            return trim(stripcslashes($text));
        }

        return $text;
    }

    protected $meta_description = null;
    protected $og_meta_desc = null;

    /**
     * Creates the meta description element
     * @param bool $show
     * @return null|string
     */
    private function get_meta_description( $show = true )
    {
        if (is_null($this->meta_description)) {
            $this->generate_meta_description();
        }

        if($show !== false) {
            if (is_string($this->meta_description) && $this->meta_description !== '') {
                return sprintf('<meta name="description" content="%s" />', esc_attr(wp_strip_all_tags(stripslashes($this->meta_description)))) . "\n";
            }
        }
        return $this->meta_description;
    }

    private function get_og_description()
    {
        if (is_null($this->og_meta_desc)) {
            $this->generate_og_description();
        }

        if (is_string($this->og_meta_desc) && $this->og_meta_desc !== '') {
            return esc_attr(wp_strip_all_tags(stripslashes($this->og_meta_desc)));
        }

        return $this->og_meta_desc;
    }

    /**
     * Return the default meta description from the post if open graph description field is empty
     * If both description fields are empty, post excerpt or post content overwrites the description tag
     *
     * @return string
     */
    private function generate_meta_description()
    {
        global $post;
        $wtt_settings = get_option('wtt_settings');
        $description = '';
        $main_desc = false;
        $desc_template = '';
        $post_type = '';

        if (is_object($post) && (isset($post->post_type) && $post->post_type !== '')) {
            $post_type = $post->post_type;
        }

        if ($this->is_woocommerce_page()) {
            $shop_page_id = function_exists( 'wc_get_page_id' ) ? wc_get_page_id( 'shop' ) : ( -1 );
            $post = get_post($shop_page_id);
            $post_type = get_query_var('post_type');

            $option_key = 'metadesc-ptarchive-'.$post_type;
            if ( ( $description === '' && $post_type !== '' ) && isset( $wtt_settings[ $option_key ] ) ) {
                $desc_template = $wtt_settings[ 'metadesc-ptarchive-' . $post_type ];
            }
            $description = $this->getPostMeta( '_wtt_post_description', $post->ID );
        }
        elseif ($this->is_single_page()) {
            $post      = get_post( $this->get_single_page_id() );
            $post_type = $post->post_type;

            $option_key = 'metadesc-' . $post_type;
            if ( ( $description === '' && $post_type !== '' ) && isset( $wtt_settings[ $option_key ] ) ) {
                $desc_template = $wtt_settings[ $option_key ];
            }
            $main_desc = $this->getPostMeta( '_wtt_post_description', $post->ID );
        }
        else {
            if (is_search()) {
                $description = '';
            } elseif (get_option('show_on_front') === 'posts' && is_home()) { //Home post page
                $desc_template = $wtt_settings['metadesc-home-wtt'];
                if (empty($desc_template)) {
                    $desc_template = get_bloginfo('description');
                }
            } elseif (get_option('show_on_front') === 'page' && is_page(get_option('page_on_front')) && is_front_page()) { //Home static page
                $description = $this->getPostMeta('_wtt_post_description');
                if (($description === '' && $post_type !== '') && isset($wtt_settings['metadesc-' . $post_type])) {
                    $desc_template = $wtt_settings['metadesc-' . $post_type];
                }
            } elseif (is_category() || is_tag() || is_tax()) {
                $term = $GLOBALS['wp_query']->get_queried_object();
                $description = $this->get_wtt_tax_term_meta($term, $term->taxonomy, '_wtt_term_description');

                if (is_object($term) && isset($term->taxonomy) && isset($wtt_settings['metadesc-tax-' . $term->taxonomy]) && $wtt_settings['metadesc-tax-' . $term->taxonomy] !== '') {
                    $desc_template = $wtt_settings['metadesc-tax-' . $term->taxonomy];
                }
            } elseif (is_author()) {
                if (isset($wtt_settings['metadesc-author-wtt']) && $wtt_settings['metadesc-author-wtt'] !== '') {
                    $desc_template = $wtt_settings['metadesc-author-wtt'];
                }
            } elseif (is_post_type_archive()) {
                if (isset($wtt_settings['metadesc-ptarchive-' . $post_type])) {
                    $desc_template = $wtt_settings['metadesc-ptarchive-' . $post_type];
                }
            } elseif (is_archive()) {
                $desc_template = isset($wtt_settings['metadesc-archive-wtt']) ? $wtt_settings['metadesc-archive-wtt'] : '';
            }
        }

        if ($main_desc !== '' && is_string($main_desc)) {
            $description = $main_desc;
        } elseif ($desc_template !== '' && ($description === '')) {
            $description = $desc_template;
        }

        $description = $this->replacePatterns($description, $post);

        $this->meta_description = $description;
    }

    private function generate_og_description()
    {
        global $post;
        $og_desc = '';

        if (is_front_page()) {
            $og_desc = $this->meta_description;
        }

        if ($this->is_single_page()) {
            $postId = $this->get_single_page_id();
            $post = get_post( $postId );
            $og_desc = self::getPostMeta('_wtt_opengraph-description');

            $og_desc = $this->replacePatterns( $og_desc, $post );

            if ($og_desc === '') {
                $og_desc = $this->get_meta_description( false );
            }

            if (!is_string($og_desc) || (is_string($og_desc) && $og_desc === '')) {
                $og_desc = $this->truncateDesc($post->post_excerpt, 0, 200);

                $og_desc = str_replace( '[&hellip;]', '&hellip;', wp_trim_words(get_post_field('post_content', $postId), 55));
            }
        }

        if (is_tax() || is_category() || is_tag()) {
            $og_desc = '';
            if ($og_desc === '') {
                $og_desc = $this->get_meta_description(false);
            }
            if ($og_desc === '') {
                $og_desc = wp_strip_all_tags(term_description());
            }
            if ( $og_desc === '' ) {
                $og_desc = $this->get_wtt_meta_without_term('_wtt_term_description');
            }
        }

        if ( is_string( $og_desc ) && $og_desc !== '' ) {
            $this->og_meta_desc = trim(strip_shortcodes($og_desc));
        }
    }

    protected $canonical_url;
    protected $og_url;

    /**
     * Return the default canonical url from the post if canonical field is empty
     *
     * @return string
     */
    private function get_canonical_url()
    {
        if (is_null($this->canonical_url)) {
            $this->generate_canonical_url();
        }

        if (is_string($this->canonical_url) && $this->canonical_url !== '') {
            return sprintf('<link rel="canonical" href="%s" />', esc_url($this->canonical_url)) . "\n";
        }

        return $this->canonical_url;
    }

    private function get_og_url()
    {
        if (is_null($this->og_url)) {
            $this->generate_canonical_url();
        }

        if (is_string($this->og_url) && $this->og_url !== '') {
            return esc_url($this->og_url);
        }

        return $this->og_url;
    }

    private function generate_canonical_url()
    {
        global $post;
        $url = "";
        $wtt_canonical_url = '';

        if(is_singular()) {
            $queried_object = get_queried_object();
            $url = get_permalink( $queried_object->ID );

            $wtt_canonical_url = self::getPostMeta('_wtt_canonical-link');

            if ( $post->ID === get_queried_object_id() ) {
                $page = get_query_var( 'page', 0 );
                if ( $page >= 2 ) {
                    if ( '' == get_option( 'permalink_structure' ) ) {
                        $url = add_query_arg( 'page', $page, $url );
                    } else {
                        $url = trailingslashit( $url ) . user_trailingslashit( $page, 'single_paged' );
                    }
                }

                $cpage = get_query_var( 'cpage', 0 );
                if ( $cpage ) {
                    $url = get_comments_pagenum_link( $cpage );
                }
            }
        } else {
            if (is_search()) {
                $search_query = get_search_query();
                if (!empty($search_query) && !preg_match('|^page/\d+$|', $search_query)) {
                    $url = get_search_link();
                }
            } elseif(is_front_page()) {
                $url = self::get_home_url();
            } elseif (is_home() && get_option( 'show_on_front' ) === 'page') {

                $posts_page_id = get_option( 'page_for_posts' );
                $url = $this->getPostMeta( '_wtt_canonical-link', $posts_page_id );

                if ( empty( $url ) ) {
                    $url = get_permalink( $posts_page_id );
                }
            } elseif(is_category() || is_tag() || is_tax()) {
                $term = $GLOBALS['wp_query']->get_queried_object();

                if(!empty($term)) {
                    $term_link = get_term_link($term, $term->taxonomy);

                    if ( !is_wp_error( $term_link ) ) {
                        $url = $term_link;
                    }
                }
            } elseif ( is_post_type_archive() ) {
                $post_type = get_query_var('post_type');
                $url = get_post_type_archive_link( $post_type );
            }
            elseif ( is_author() ) {
                $url = get_author_posts_url( get_query_var( 'author' ), get_query_var( 'author_name' ) );
            }
            elseif ( is_archive() ) {
                if ( is_date() ) {
                    if ( is_day() ) {
                        $url = get_day_link( get_query_var( 'year' ), get_query_var( 'monthnum' ), get_query_var( 'day' ) );
                    }
                    elseif ( is_month() ) {
                        $url = get_month_link( get_query_var( 'year' ), get_query_var( 'monthnum' ) );
                    }
                    elseif ( is_year() ) {
                        $url = get_year_link( get_query_var( 'year' ) );
                    }
                }
            }
        }

        if (is_string( $wtt_canonical_url ) && $wtt_canonical_url !== '' ) {
            $url = $wtt_canonical_url;
        }

        $this->canonical_url = $url;
        $this->og_url = $url;
    }

    public static function get_home_url($url_path = '') {

        $url = home_url();

        if ( !empty( $url_path ) ) {
            return $url;
        }

        $url_path = wp_parse_url( $url, PHP_URL_PATH );

        if ( '/' === $url_path ) {
            return $url;
        }

        if ( is_null( $url_path ) ) {
            return trailingslashit( $url );
        }

        if ( is_string( $url_path ) ) {
            return user_trailingslashit( $url );
        }

        return $url;
    }

    /**
     * Return the default title from the post if og title is empty
     *
     * @return string
     */
    private function generate_og_title()
    {
        if($this->is_single_page()) {
            $post_id = $this->get_single_page_id();
            $post = get_post($post_id);
            $title = $this->getPostMeta('_wtt_opengraph-title', $post_id);

            if($title === '' || $title === false) {
                $title = $this->wtt_generate_seo_title('', "");
            } else {
                $title = $this->replacePatterns($title, $post);
            }
        }
        elseif(is_front_page() ) {
            $title = $this->wtt_generate_seo_title('', "");
        }
        elseif(is_tag() || is_tax() || is_category()) {
            $title = $this->get_wtt_meta_without_term('_wtt_term_title');

            if($title === '') {
                $title = $this->wtt_generate_seo_title('', "");
            } else {
                $title = $this->replacePatterns($title, $GLOBALS['wp_query']->get_queried_object());
            }
        }
        else {
            $title = $this->wtt_generate_seo_title('', "");
        }

        return trim($title);
    }

    /**
     * Get image for meta data
     *
     * @param null $post_id
     * @return false|mixed|string
     */
    private function get_custom_image($post_id = null)
    {
        $imageIsDone = false;
        $openGraphImage = '';
        $wtt_social = get_option("wtt_social");

        if ($post_id) {
            $current_post = false;
            //Specific post
            $post = get_post($post_id);
        } else {
            $current_post = true;
            //Current post
            global $post;
        }
        if ($post) {
            /**
             * If page is attachment
             */
            if (!$current_post && is_attachment()) {
                if ($temp = wp_get_attachment_image_src(null, 'full')) {
                    $openGraphImage = trim($temp[0]);
                    if (trim($openGraphImage) != '') {
                        $imageIsDone = true;
                    }
                }
            }

            /**
             * If specific image from post
             */
            if (!$imageIsDone) {
                if (!empty($wtt_social['og_image_use_specific']) && $wtt_social['og_image_use_specific'] == "on") {
                    if ($openGraphImage = self::getPostMeta('_wtt_opengraph-image')) {
                        if ($openGraphImage != '') {
                            $imageIsDone = true;
                        }
                    }
                }
            }

            /**
             * If featured image from post is active
             */
            if (!$imageIsDone) {
                if (function_exists('get_post_thumbnail_id')) {
                    if (!empty($wtt_social['og_image_use_featured']) && $wtt_social['og_image_use_featured'] == "on") {
                        if ($id_attachment = get_post_thumbnail_id($post->ID)) {
                            $openGraphImage = wp_get_attachment_url($id_attachment);
                            $imageIsDone = true;
                        }
                    }
                }
            }

            /**
             * If use default image specified in the settings screen
             */
            if (!$imageIsDone) {
                if (!empty($wtt_social['og_image_use_default']) && $wtt_social['og_image_use_default'] == "on") {
                    $openGraphImage = $wtt_social['opengraph_image'];
                } else {
                    $openGraphImage = '';
                }
            }

            return $openGraphImage;
        } else {
            //No post
            return false;
        }
    }

    /**
     * Hooks into the save post function and saves wtt social meta tags
     *
     * @param $post_id
     * @return mixed
     */
    function wtt_save_social_meta($post_id)
    {
        // Check if our nonce is set.
        if (!isset($_POST['wttcontent']))
            return $post_id;

        $nonce = $_POST['wttcontent'];

        // Verify that the nonce is valid.
        if (!wp_verify_nonce($nonce, 'wttcallback'))
            return $post_id;

        // If this is an autosave, our form has not been submitted, so we don't want to do anything.
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
            return $post_id;

        // Check the user's permissions.
        if ('page' == $_POST['post_type']) {
            if (!current_user_can('edit_page', $post_id))
                return $post_id;
        } else {
            if (!current_user_can('edit_post', $post_id))
                return $post_id;
        }

        // Sanitize user input.
        $openGraphTitle = stripslashes($_POST['wtt_opengraph-title']);
        $openGraphDescription = stripslashes(str_replace(array("\r\n", "\r", "\n"), '', $_POST['wtt_opengraph-description']));
        $openGraphImage = sanitize_text_field($_POST['wtt_opengraph-image']);
        $canonicalLink = sanitize_text_field($_POST['wtt_canonical-link']);

        // Update the meta field in the database if the field is not empty.
        if (isset($_POST['wtt_opengraph-title']) && ($_POST['wtt_opengraph-title']) <> "") {
            update_post_meta($post_id, '_wtt_opengraph-title', $openGraphTitle);
        } else {
            delete_post_meta($post_id, '_wtt_opengraph-title', $openGraphTitle);
        }

        if (isset($_POST['wtt_opengraph-description']) && ($_POST['wtt_opengraph-description']) <> "") {
            update_post_meta($post_id, '_wtt_opengraph-description', $openGraphDescription);
        } else {
            delete_post_meta($post_id, '_wtt_opengraph-description', $openGraphDescription);
        }

        if (isset($_POST['wtt_opengraph-image']) && ($_POST['wtt_opengraph-image']) <> "") {
            update_post_meta($post_id, '_wtt_opengraph-image', $openGraphImage);
        } else {
            delete_post_meta($post_id, '_wtt_opengraph-image', $openGraphImage);
        }

        if (isset($_POST['wtt_canonical-link']) && ($_POST['wtt_canonical-link']) <> "") {
            update_post_meta($post_id, '_wtt_canonical-link', $canonicalLink);
        } else {
            delete_post_meta($post_id, '_wtt_canonical-link', $canonicalLink);
        }

        return $post_id;
    }

    /**
     * If wp seo is active, return true
     *
     * @return bool
     */
    public function is_wp_seo_active()
    {
        if (defined('WPSEO_VERSION')) {
            return true;
        }
        return false;
    }
}